'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (_ref) {
  var t = _ref.types;

  var findStyles = function findStyles(children) {
    return children.filter(function (el) {
      return t.isJSXElement(el) && el.openingElement.name.name === 'style' && el.openingElement.attributes.some(function (attr) {
        return attr.name.name === STYLE_ATTRIBUTE;
      });
    });
  };

  var getExpressionText = function getExpressionText(expr) {
    return t.isTemplateLiteral(expr) ? expr.quasis[0].value.raw :
    // assume string literal
    expr.value;
  };

  return {
    inherits: _babelPluginSyntaxJsx2.default,
    visitor: {
      JSXOpeningElement: function JSXOpeningElement(path, state) {
        if (state.hasJSXStyle) {
          if (state.ignoreClosing == null) {
            // this flag has a two-fold purpose:
            // - ignore the opening tag of the parent element
            //   of the style tag, since we don't want to add
            //   the attribute to that one
            // - keep a counter of elements inside so that we
            //   can keep track of when we exit the parent
            //   to reset state
            state.ignoreClosing = 1;
            return;
          }

          var el = path.node;

          if (el.name && el.name.name !== 'style') {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
              for (var _iterator = el.attributes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var _attr = _step.value;

                if (_attr.name === MARKUP_ATTRIBUTE) {
                  // avoid double attributes
                  return;
                }
              }
            } catch (err) {
              _didIteratorError = true;
              _iteratorError = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }
              } finally {
                if (_didIteratorError) {
                  throw _iteratorError;
                }
              }
            }

            var attr = t.jSXAttribute(t.JSXIdentifier(MARKUP_ATTRIBUTE), t.JSXExpressionContainer(t.stringLiteral(state.jsxId)));
            el.attributes.push(attr);
          }

          state.ignoreClosing++;
          // next visit will be: JSXElement exit()
        }
      },

      JSXElement: {
        enter: function enter(path, state) {
          if (state.hasJSXStyle == null) {
            var styles = findStyles(path.node.children);

            if (styles.length > 0) {
              state.jsxId = '';
              state.styles = [];
              var _iteratorNormalCompletion2 = true;
              var _didIteratorError2 = false;
              var _iteratorError2 = undefined;

              try {
                for (var _iterator2 = styles[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                  var style = _step2.value;

                  if (style.children.length !== 1) {
                    throw path.buildCodeFrameError('Expected a child under ' + ('JSX Style tag, but got ' + style.children.length + ' ') + '(eg: <style jsx>{`hi`}</style>)');
                  }

                  var child = style.children[0];

                  if (!t.isJSXExpressionContainer(child)) {
                    throw path.buildCodeFrameError('Expected a child of ' + 'type JSXExpressionContainer under JSX Style tag ' + ('(eg: <style jsx>{`hi`}</style>), got ' + child.type));
                  }

                  var expression = child.expression;

                  if (!t.isTemplateLiteral(child.expression) && !t.isStringLiteral(child.expression)) {
                    throw path.buildCodeFrameError('Expected a template ' + 'literal or String literal as the child of the ' + 'JSX Style tag (eg: <style jsx>{`some css`}</style>),' + (' but got ' + expression.type));
                  }

                  var styleText = getExpressionText(expression);
                  var styleId = String((0, _murmurhash2.default)(styleText));
                  state.styles.push([styleId, styleText]);
                }
              } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                  }
                } finally {
                  if (_didIteratorError2) {
                    throw _iteratorError2;
                  }
                }
              }

              state.jsxId += (0, _murmurhash2.default)(state.styles.map(function (s) {
                return s[1];
              }).join(''));
              state.hasJSXStyle = true;
              state.file.hasJSXStyle = true;
              // next visit will be: JSXOpeningElement
            } else {
              state.hasJSXStyle = false;
            }
          } else if (state.hasJSXStyle) {
            var el = path.node.openingElement;

            if (el.name && el.name.name === 'style') {
              // we replace styles with the function call
              var _state$styles$shift = state.styles.shift(),
                  _state$styles$shift2 = _slicedToArray(_state$styles$shift, 2),
                  id = _state$styles$shift2[0],
                  css = _state$styles$shift2[1];

              path.replaceWith(t.JSXExpressionContainer(t.callExpression(t.identifier(INJECT_METHOD), [t.stringLiteral(id), t.stringLiteral((0, _styleTransform2.default)(id, css))])));
            }
          }
        },
        exit: function exit(path, state) {
          if (state.hasJSXStyle && ! --state.ignoreClosing) {
            state.hasJSXStyle = null;
          }
        }
      },
      Program: {
        enter: function enter(path, state) {
          state.file.hasJSXStyle = false;
        },
        exit: function exit(_ref2, state) {
          var node = _ref2.node,
              scope = _ref2.scope;

          if (!(state.file.hasJSXStyle && !scope.hasBinding(INJECT_METHOD))) {
            return;
          }

          var importDeclaration = t.importDeclaration([t.importDefaultSpecifier(t.identifier(INJECT_METHOD))], t.stringLiteral('styled-jsx/inject'));

          node.body.unshift(importDeclaration);
        }
      }
    }
  };
};

var _babelPluginSyntaxJsx = require('babel-plugin-syntax-jsx');

var _babelPluginSyntaxJsx2 = _interopRequireDefault(_babelPluginSyntaxJsx);

var _murmurhash = require('../lib/murmurhash2');

var _murmurhash2 = _interopRequireDefault(_murmurhash);

var _styleTransform = require('../lib/style-transform');

var _styleTransform2 = _interopRequireDefault(_styleTransform);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var STYLE_ATTRIBUTE = 'jsx';
var MARKUP_ATTRIBUTE = 'data-jsx';
var INJECT_METHOD = '_jsxStyleInject';